@@ -81,6 +81,9 @@ typedef struct {
 
   TpAccountManager *account_manager;
 
+  GtkWidget *param_account_widget;
+  GtkWidget *param_password_widget;
+
   gboolean dispose_run;
 } EmpathyAccountWidgetPriv;
 
@@ -168,12 +171,8 @@ static void
 account_widget_entry_changed_cb (GtkEditable *entry,
     EmpathyAccountWidget *self)
 {
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-
   account_widget_entry_changed_common (self, GTK_ENTRY (entry), FALSE);
-  account_widget_handle_control_buttons_sensitivity (self);
-
-  priv->contains_pending_changes = TRUE;
+  empathy_account_widget_changed (self);
 }
 
 static void
@@ -214,8 +213,7 @@ account_widget_int_changed_cb (GtkWidget *widget,
       g_return_if_reached ();
     }
 
-  account_widget_handle_control_buttons_sensitivity (self);
-  priv->contains_pending_changes = TRUE;
+  empathy_account_widget_changed (self);
 }
 
 static void
@@ -247,8 +245,7 @@ account_widget_checkbutton_toggled_cb (GtkWidget *widget,
       empathy_account_settings_set_boolean (priv->settings, param_name, value);
     }
 
-  account_widget_handle_control_buttons_sensitivity (self);
-  priv->contains_pending_changes = TRUE;
+  empathy_account_widget_changed (self);
 }
 
 static void
@@ -265,8 +262,7 @@ account_widget_forget_clicked_cb (GtkWidget *button,
   empathy_account_settings_unset (priv->settings, param_name);
   gtk_entry_set_text (GTK_ENTRY (priv->entry_password), "");
 
-  account_widget_handle_control_buttons_sensitivity (self);
-  priv->contains_pending_changes = TRUE;
+  empathy_account_widget_changed (self);
 }
 
 static void
@@ -345,8 +341,7 @@ account_widget_combobox_changed_cb (GtkWidget *widget,
       empathy_account_settings_set_string (priv->settings, param_name, value);
     }
 
-  account_widget_handle_control_buttons_sensitivity (self);
-  priv->contains_pending_changes = TRUE;
+  empathy_account_widget_changed (self);
 }
 
 void
@@ -405,6 +400,11 @@ empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
       str = empathy_account_settings_get_string (priv->settings, param_name);
       gtk_entry_set_text (GTK_ENTRY (widget), str ? str : "");
 
+      if (!tp_strdiff (param_name, "account"))
+        priv->param_account_widget = widget;
+      else if (!tp_strdiff (param_name, "password"))
+        priv->param_password_widget = widget;
+
       if (strstr (param_name, "password"))
         {
           gtk_entry_set_visibility (GTK_ENTRY (widget), FALSE);
@@ -852,10 +852,12 @@ account_widget_build_salut (EmpathyAccountWidget *self,
     const char *filename)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  GtkWidget *expander_advanced;
 
   self->ui_details->gui = empathy_builder_get_file (filename,
       "table_common_settings", &priv->table_common_settings,
       "vbox_salut_settings", &self->ui_details->widget,
+      "expander_advanced_settings", &expander_advanced,
       NULL);
 
   empathy_account_widget_handle_params (self,
@@ -867,6 +869,9 @@ account_widget_build_salut (EmpathyAccountWidget *self,
       "entry_jid", "jid",
       NULL);
 
+  if (priv->simple)
+    gtk_widget_hide (expander_advanced);
+
   self->ui_details->default_focus = g_strdup ("entry_first_name");
 }
 
@@ -1311,6 +1316,10 @@ presence_changed_cb (TpAccountManager *manager,
   if (priv->destroyed)
     return;
 
+  if (priv->apply_button == NULL)
+    /* This button doesn't exist in 'simple' mode */
+    return;
+
   if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
     {
       /* We are online, display a Login button */
@@ -1456,6 +1465,9 @@ add_register_buttons (EmpathyAccountWidget *self,
   if (account_widget_is_gtalk (self))
     return;
 
+  if (priv->simple)
+    return;
+
   priv->radiobutton_reuse = gtk_radio_button_new_with_label (NULL,
       _("This account already exists on the server"));
   radiobutton_register = gtk_radio_button_new_with_label (
@@ -1870,3 +1882,37 @@ empathy_account_widget_get_default_display_name (EmpathyAccountWidget *self)
 
   return default_display_name;
 }
+
+/* Used by subclass to indicate that widget contains pending changes */
+void
+empathy_account_widget_changed (EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  account_widget_handle_control_buttons_sensitivity (self);
+  priv->contains_pending_changes = TRUE;
+}
+
+void
+empathy_account_widget_set_account_param (EmpathyAccountWidget *self,
+    const gchar *account)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (priv->param_account_widget == NULL)
+    return;
+
+  gtk_entry_set_text (GTK_ENTRY (priv->param_account_widget), account);
+}
+
+void
+empathy_account_widget_set_password_param (EmpathyAccountWidget *self,
+    const gchar *account)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (priv->param_password_widget == NULL)
+    return;
+
+  gtk_entry_set_text (GTK_ENTRY (priv->param_password_widget), account);
+}
