@@ -1112,7 +1112,6 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	gboolean                    do_set_active = FALSE;
 	gboolean                    do_set_refresh = FALSE;
 	gboolean                    show_avatar = FALSE;
-	gboolean                    show_protocol = FALSE;
 	GdkPixbuf                  *pixbuf_avatar;
 	GdkPixbuf                  *pixbuf_status;
 
@@ -1207,9 +1206,6 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	if (priv->show_avatars && !priv->is_compact) {
 		show_avatar = TRUE;
 	}
-	if (priv->show_protocols && !priv->is_compact) {
-		show_protocol = TRUE;
-	}
 	pixbuf_avatar = empathy_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
 	pixbuf_status = contact_list_store_get_contact_status_icon (store, contact);
 	for (l = iters; l && set_model; l = l->next) {
@@ -1645,7 +1641,6 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 {
 	EmpathyContactListStorePriv *priv;
 	gboolean                     show_avatar = FALSE;
-	gboolean                     show_protocol = FALSE;
 	EmpathyContact              *contact;
 	GdkPixbuf                   *pixbuf_status;
 
@@ -1654,9 +1649,6 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 	if (priv->show_avatars && !priv->is_compact) {
 		show_avatar = TRUE;
 	}
-	if (priv->show_protocols && !priv->is_compact) {
-		show_protocol = TRUE;
-	}
 
 	gtk_tree_model_get (model, iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
@@ -1687,14 +1679,10 @@ contact_list_store_get_contact_status_icon_with_icon_name (
 	EmpathyContactListStorePriv *priv;
 	const gchar                 *protocol_name = NULL;
 	gchar                       *icon_name = NULL;
-	gboolean                     show_protocol = FALSE;
 
 	priv = GET_PRIV (store);
 
-	if (priv->show_protocols && !priv->is_compact) {
-		show_protocol = TRUE;
-	}
-	if (show_protocol) {
+	if (priv->show_protocols) {
 		protocol_name = empathy_protocol_name_for_contact (contact);
 		icon_name = g_strdup_printf ("%s-%s", status_icon_name, protocol_name);
 	} else {
@@ -1704,7 +1692,7 @@ contact_list_store_get_contact_status_icon_with_icon_name (
 	if (pixbuf_status == NULL) {
 		pixbuf_status = empathy_pixbuf_contact_status_icon_with_icon_name (contact,
 				    status_icon_name,
-				    show_protocol);
+				    priv->show_protocols);
 		if (pixbuf_status != NULL) {
 			g_hash_table_insert (priv->status_icons,
 			    g_strdup (icon_name),
