@@ -175,6 +175,8 @@ contact_list_view_query_tooltip_cb (EmpathyContactListView *view,
 		priv->tooltip_widget = empathy_contact_widget_new (contact,
 			EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP |
 			EMPATHY_CONTACT_WIDGET_SHOW_LOCATION);
+		gtk_container_set_border_width (
+			GTK_CONTAINER (priv->tooltip_widget), 8);
 		g_object_ref (priv->tooltip_widget);
 		g_signal_connect (priv->tooltip_widget, "destroy",
 				  G_CALLBACK (contact_list_view_tooltip_destroy_cb),
@@ -752,54 +754,49 @@ contact_list_view_row_activated (GtkTreeView       *view,
 }
 
 static void
-contact_list_start_voip_call (EmpathyCellRendererActivatable *cell,
-    const gchar                    *path_string,
-    EmpathyContactListView         *view,
-    gboolean with_video)
-{
-	EmpathyContactListViewPriv *priv = GET_PRIV (view);
-	GtkTreeModel               *model;
-	GtkTreeIter                 iter;
-	EmpathyContact             *contact;
-
-	if (!(priv->contact_features & EMPATHY_CONTACT_FEATURE_CALL)) {
-		return;
-	}
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-	if (!gtk_tree_model_get_iter_from_string (model, &iter, path_string)) {
-		return;
-	}
-
-	gtk_tree_model_get (model, &iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
-			    -1);
-
-	if (contact) {
-		EmpathyCallFactory *factory;
-		factory = empathy_call_factory_get ();
-		empathy_call_factory_new_call_with_streams (factory, contact,
-			TRUE, with_video);
-		g_object_unref (contact);
-	}
-}
-
-static void
-contact_list_view_video_call_activated_cb (
+contact_list_view_call_activated_cb (
     EmpathyCellRendererActivatable *cell,
     const gchar                    *path_string,
     EmpathyContactListView         *view)
 {
-  contact_list_start_voip_call (cell, path_string, view, TRUE);
-}
+	GtkWidget *menu;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	EmpathyContact *contact;
+	GdkEventButton *event;
+	GtkMenuShell *shell;
+	GtkWidget *item;
 
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+	if (!gtk_tree_model_get_iter_from_string (model, &iter, path_string))
+		return;
 
-static void
-contact_list_view_audio_call_activated_cb (EmpathyCellRendererActivatable *cell,
-				     const gchar                    *path_string,
-				     EmpathyContactListView         *view)
-{
-  contact_list_start_voip_call (cell, path_string, view, FALSE);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
+	if (contact == NULL)
+		return;
+
+	event = (GdkEventButton *) gtk_get_current_event ();
+
+	menu = gtk_menu_new ();
+	shell = GTK_MENU_SHELL (menu);
+
+	/* audio */
+	item = empathy_contact_audio_call_menu_item_new (contact);
+	gtk_menu_shell_append (shell, item);
+	gtk_widget_show (item);
+
+	/* video */
+	item = empathy_contact_video_call_menu_item_new (contact);
+	gtk_menu_shell_append (shell, item);
+	gtk_widget_show (item);
+
+	gtk_widget_show (menu);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+			event->button, event->time);
+
+	g_object_unref (contact);
 }
 
 static void
@@ -874,49 +871,23 @@ contact_list_view_audio_call_cell_data_func (
 {
 	gboolean is_group;
 	gboolean is_active;
-	gboolean can_voip;
+	gboolean can_audio, can_video;
 
 	gtk_tree_model_get (model, iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
-			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL, &can_voip,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL, &can_audio,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL, &can_video,
 			    -1);
 
 	g_object_set (cell,
-		      "visible", !is_group && can_voip,
-		      "icon-name", EMPATHY_IMAGE_VOIP,
+		      "visible", !is_group && (can_audio || can_video),
+		      "icon-name", can_video? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
 		      NULL);
 
 	contact_list_view_cell_set_background (view, cell, is_group, is_active);
 }
 
-static void
-contact_list_view_video_call_cell_data_func (
-				       GtkTreeViewColumn      *tree_column,
-				       GtkCellRenderer        *cell,
-				       GtkTreeModel           *model,
-				       GtkTreeIter            *iter,
-				       EmpathyContactListView *view)
-{
-	gboolean is_group;
-	gboolean is_active;
-	gboolean can_voip;
-
-	gtk_tree_model_get (model, iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
-			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
-			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL, &can_voip,
-			    -1);
-
-	g_object_set (cell,
-		      "visible", !is_group && can_voip,
-		      "icon-name", EMPATHY_IMAGE_VIDEO_CALL,
-		      NULL);
-
-	contact_list_view_cell_set_background (view, cell, is_group, is_active);
-}
-
-
 static void
 contact_list_view_avatar_cell_data_func (GtkTreeViewColumn     *tree_column,
 					 GtkCellRenderer       *cell,
@@ -1155,23 +1126,7 @@ contact_list_view_setup (EmpathyContactListView *view)
 		      NULL);
 
 	g_signal_connect (cell, "path-activated",
-			  G_CALLBACK (contact_list_view_audio_call_activated_cb),
-			  view);
-
-	/* Video Call Icon */
-	cell = empathy_cell_renderer_activatable_new ();
-	gtk_tree_view_column_pack_start (col, cell, FALSE);
-	gtk_tree_view_column_set_cell_data_func (
-		col, cell,
-		(GtkTreeCellDataFunc) contact_list_view_video_call_cell_data_func,
-		view, NULL);
-
-	g_object_set (cell,
-		      "visible", FALSE,
-		      NULL);
-
-	g_signal_connect (cell, "path-activated",
-			  G_CALLBACK (contact_list_view_video_call_activated_cb),
+			  G_CALLBACK (contact_list_view_call_activated_cb),
 			  view);
 
 	/* Avatar */
