@@ -25,6 +25,7 @@
 #include <glib/gi18n.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/interfaces.h>
 
 #include "empathy-ft-handler.h"
 #include "empathy-tp-contact-factory.h"
@@ -819,6 +820,7 @@ hash_job_done (gpointer user_data)
   if (hash_data->error != NULL)
     {
       error = hash_data->error;
+      hash_data->error = NULL;
       goto cleanup;
     }
 
@@ -1218,7 +1220,7 @@ contact_factory_contact_cb (EmpathyTpContactFactory *factory,
       return;
     }
 
-  priv->contact = contact;
+  priv->contact = g_object_ref (contact);
 
   cb_data->callback (handler, NULL, cb_data->user_data);
 }
