@@ -35,14 +35,14 @@
 #include <glib/gi18n.h>
 #include <libnotify/notification.h>
 
-#include <telepathy-glib/account-manager.h>
-#include <telepathy-glib/util.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
-#include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-tp-contact-factory.h>
+#include <libempathy/empathy-contact-list.h>
 
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-conf.h>
@@ -56,6 +56,7 @@
 
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
+#include "empathy-invite-participant-dialog.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -92,6 +93,7 @@ typedef struct {
 	GtkAction   *menu_edit_cut;
 	GtkAction   *menu_edit_copy;
 	GtkAction   *menu_edit_paste;
+	GtkAction   *menu_edit_find;
 
 	GtkAction   *menu_tabs_next;
 	GtkAction   *menu_tabs_prev;
@@ -251,9 +253,9 @@ chat_window_create_label (EmpathyChatWindow *window,
 
 	gtk_misc_set_padding (GTK_MISC (name_label), 2, 0);
 	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
-	g_object_set_data (G_OBJECT (chat),
+	g_object_set_data_full (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-label" : "chat-window-menu-label",
-		name_label);
+		g_object_ref (name_label), g_object_unref);
 
 	status_image = gtk_image_new ();
 
@@ -263,12 +265,12 @@ chat_window_create_label (EmpathyChatWindow *window,
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), status_image, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), name_label, TRUE, TRUE, 0);
 
-	g_object_set_data (G_OBJECT (chat),
+	g_object_set_data_full (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-image" : "chat-window-menu-image",
-		status_image);
-	g_object_set_data (G_OBJECT (chat),
+		g_object_ref (status_image), g_object_unref);
+	g_object_set_data_full (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-tooltip-widget" : "chat-window-menu-tooltip-widget",
-		event_box);
+		g_object_ref (event_box), g_object_unref);
 
 	gtk_container_add (GTK_CONTAINER (event_box), event_box_hbox);
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
@@ -276,7 +278,8 @@ chat_window_create_label (EmpathyChatWindow *window,
 	if (is_tab_label) {
 		close_button = gtk_button_new ();
 		gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
-		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
+		g_object_set_data_full (G_OBJECT (chat), "chat-window-tab-close-button",
+			g_object_ref (close_button), g_object_unref);
 
 		/* We don't want focus/keynav for the button to avoid clutter, and
 		 * Ctrl-W works anyway.
@@ -345,6 +348,32 @@ chat_window_menu_context_update (EmpathyChatWindowPriv *priv,
 	gtk_action_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
 }
 
+static void
+chat_window_conversation_menu_update (EmpathyChatWindowPriv *priv,
+                                      EmpathyChatWindow     *self)
+{
+	EmpathyTpChat *tp_chat;
+	TpConnection *connection;
+	GtkAction *action;
+	gboolean sensitive = FALSE;
+
+	g_return_if_fail (priv->current_chat != NULL);
+
+	action = gtk_ui_manager_get_action (priv->ui_manager,
+		"/chats_menubar/menu_conv/menu_conv_invite_participant");
+	tp_chat = empathy_chat_get_tp_chat (priv->current_chat);
+
+	if (tp_chat != NULL) {
+		connection = empathy_tp_chat_get_connection (tp_chat);
+
+		sensitive = empathy_tp_chat_can_add_contact (tp_chat) &&
+			(tp_connection_get_status (connection, NULL) ==
+			 TP_CONNECTION_STATUS_CONNECTED);
+	}
+
+	gtk_action_set_sensitive (action, sensitive);
+}
+
 static void
 chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 				 EmpathyChatWindow     *window)
@@ -536,6 +565,8 @@ chat_window_update (EmpathyChatWindow *window)
 	chat_window_menu_context_update (priv,
 					 num_pages);
 
+	chat_window_conversation_menu_update (priv, window);
+
 	chat_window_contact_menu_update (priv,
 					 window);
 
@@ -697,8 +728,8 @@ chat_window_chat_notify_cb (EmpathyChat *chat)
 							      chat);
 		}
 
-		g_object_set_data (G_OBJECT (chat), "chat-window-remote-contact",
-				   remote_contact);
+		g_object_set_data_full (G_OBJECT (chat), "chat-window-remote-contact",
+				   g_object_ref (remote_contact), (GDestroyNotify) g_object_unref);
 	}
 
 	chat_window_update_chat_tab (chat);
@@ -818,6 +849,69 @@ chat_window_contacts_toggled_cb (GtkToggleAction   *toggle_action,
 	empathy_chat_set_show_contacts (priv->current_chat, active);
 }
 
+static void
+got_contact_cb (EmpathyTpContactFactory *factory,
+                EmpathyContact          *contact,
+                const GError            *error,
+                gpointer                 user_data,
+                GObject                 *object)
+{
+	EmpathyTpChat *tp_chat = EMPATHY_TP_CHAT (user_data);
+
+	if (error != NULL) {
+		DEBUG ("Failed: %s", error->message);
+		return;
+	} else {
+		empathy_contact_list_add (EMPATHY_CONTACT_LIST (tp_chat),
+				contact, _("Inviting you to this room"));
+	}
+}
+
+static void
+chat_window_invite_participant_activate_cb (GtkAction         *action,
+					    EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+	GtkWidget             *dialog;
+	EmpathyTpChat         *tp_chat;
+	TpChannel             *channel;
+	int                    response;
+
+	priv = GET_PRIV (window);
+
+	g_return_if_fail (priv->current_chat != NULL);
+
+	tp_chat = empathy_chat_get_tp_chat (priv->current_chat);
+	channel = empathy_tp_chat_get_channel (tp_chat);
+
+	dialog = empathy_invite_participant_dialog_new (
+			GTK_WINDOW (priv->dialog));
+	gtk_widget_show (dialog);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	if (response == GTK_RESPONSE_ACCEPT) {
+		TpConnection *connection;
+		EmpathyTpContactFactory *factory;
+		const char *id;
+
+		id = empathy_contact_selector_dialog_get_selected (
+				EMPATHY_CONTACT_SELECTOR_DIALOG (dialog), NULL);
+		if (EMP_STR_EMPTY (id)) goto out;
+
+		connection = tp_channel_borrow_connection (channel);
+		factory = empathy_tp_contact_factory_dup_singleton (connection);
+
+		empathy_tp_contact_factory_get_from_id (factory, id,
+			got_contact_cb, tp_chat,  NULL, NULL);
+
+		g_object_unref (factory);
+	}
+
+out:
+	gtk_widget_destroy (dialog);
+}
+
 static void
 chat_window_close_activate_cb (GtkAction         *action,
 			       EmpathyChatWindow *window)
@@ -908,6 +1002,19 @@ chat_window_paste_activate_cb (GtkAction         *action,
 	empathy_chat_paste (priv->current_chat);
 }
 
+static void
+chat_window_find_activate_cb (GtkAction         *action,
+			      EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_CHAT_WINDOW (window));
+
+	priv = GET_PRIV (window);
+
+	empathy_chat_find (priv->current_chat);
+}
+
 static void
 chat_window_tabs_next_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
@@ -1712,6 +1819,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
 				       "menu_edit_paste", &priv->menu_edit_paste,
+				       "menu_edit_find", &priv->menu_edit_find,
 				       "menu_tabs_next", &priv->menu_tabs_next,
 				       "menu_tabs_prev", &priv->menu_tabs_prev,
 				       "menu_tabs_left", &priv->menu_tabs_left,
@@ -1725,11 +1833,13 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
 			      "menu_conv_toggle_contacts", "toggled", chat_window_contacts_toggled_cb,
+			      "menu_conv_invite_participant", "activate", chat_window_invite_participant_activate_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
 			      "menu_edit", "activate", chat_window_edit_activate_cb,
 			      "menu_edit_cut", "activate", chat_window_cut_activate_cb,
 			      "menu_edit_copy", "activate", chat_window_copy_activate_cb,
 			      "menu_edit_paste", "activate", chat_window_paste_activate_cb,
+			      "menu_edit_find", "activate", chat_window_find_activate_cb,
 			      "menu_tabs_next", "activate", chat_window_tabs_next_activate_cb,
 			      "menu_tabs_prev", "activate", chat_window_tabs_previous_activate_cb,
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
@@ -1848,7 +1958,7 @@ empathy_chat_window_new (void)
  * be added.
  */
 EmpathyChatWindow *
-empathy_chat_window_get_default (void)
+empathy_chat_window_get_default (gboolean room)
 {
 	GList    *l;
 	gboolean  separate_windows = TRUE;
@@ -1863,13 +1973,26 @@ empathy_chat_window_get_default (void)
 	}
 
 	for (l = chat_windows; l; l = l->next) {
+		EmpathyChatWindowPriv *priv;
 		EmpathyChatWindow *chat_window;
 		GtkWidget         *dialog;
 
 		chat_window = l->data;
+		priv = GET_PRIV (chat_window);
 
 		dialog = empathy_chat_window_get_dialog (chat_window);
 		if (empathy_window_get_is_visible (GTK_WINDOW (dialog))) {
+			guint nb_rooms, nb_private;
+			empathy_chat_window_get_nb_chats (chat_window, &nb_rooms, &nb_private);
+
+			/* Skip the window if there aren't any rooms in it */
+			if (room && nb_rooms == 0)
+				continue;
+
+			/* Skip the window if there aren't any 1-1 chats in it */
+			if (!room && nb_private == 0)
+				continue;
+
 			/* Found a visible window on this desktop */
 			return chat_window;
 		}
@@ -1898,6 +2021,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	GtkWidget             *label;
 	GtkWidget             *popup_label;
 	GtkWidget             *child;
+	GValue                value = { 0, };
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
@@ -1919,6 +2043,9 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 		if (separate_windows) {
 			name = empathy_chat_get_id (chat);
 		}
+		else if (empathy_chat_is_room (chat)) {
+			name = "room-window";
+		}
 
 		empathy_geometry_bind (GTK_WINDOW (priv->dialog), name);
 	}
@@ -1942,8 +2069,13 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	gtk_notebook_append_page_menu (GTK_NOTEBOOK (priv->notebook), child, label, popup_label);
 	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
-	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (priv->notebook), child,
-					    TRUE, TRUE, GTK_PACK_START);
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+	gtk_container_child_set_property (GTK_CONTAINER (priv->notebook),
+					  child, "tab-expand" , &value);
+	gtk_container_child_set_property (GTK_CONTAINER (priv->notebook),
+					  child,  "tab-fill" , &value);
+	g_value_unset (&value);
 
 	DEBUG ("Chat added (%d references)", G_OBJECT (chat)->ref_count);
 }
@@ -2086,7 +2218,7 @@ empathy_chat_window_present_chat (EmpathyChat *chat)
 
 	/* If the chat has no window, create one */
 	if (window == NULL) {
-		window = empathy_chat_window_get_default ();
+		window = empathy_chat_window_get_default (empathy_chat_is_room (chat));
 		if (!window) {
 			window = empathy_chat_window_new ();
 		}
@@ -2101,3 +2233,24 @@ empathy_chat_window_present_chat (EmpathyChat *chat)
  	gtk_widget_grab_focus (chat->input_text_view);
 }
 
+void
+empathy_chat_window_get_nb_chats (EmpathyChatWindow *self,
+			       guint *nb_rooms,
+			       guint *nb_private)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (self);
+	GList *l;
+	guint _nb_rooms = 0, _nb_private = 0;
+
+	for (l = priv->chats; l != NULL; l = g_list_next (l)) {
+		if (empathy_chat_is_room (EMPATHY_CHAT (l->data)))
+			_nb_rooms++;
+		else
+			_nb_private++;
+	}
+
+	if (nb_rooms != NULL)
+		*nb_rooms = _nb_rooms;
+	if (nb_private != NULL)
+		*nb_private = _nb_private;
+}
